# AddComment.pl
# Add Comment v1.5 plugin for Movable Type
# http://www.chipple.net/mt/addcomment/
# by Patrick Benny http://www.chipple.net/
# This code cannot be redistributed without permission from the author. 
#
# Version history
#   1.5  - 2007/2/28 - Added optional "Date" field allowing to specify the comment or trackback date/time
#   1.4b - 2006/7/18 - Fixes double declaration that generated warning
#   1.4  - 2006/7/10 - Added "Add A Trackback" form to trackbacks tab
#   1.3  - 2006/7/6  - Added support for Movable Type 3.3
#   1.2  - 2006/7/5  - Silent rebuild, redirect to comments tab
#   1.1  - 2006/6/22 - Added "Set above as [entry's author]" button
#   1.0  - 2006/6/18 - First public release


package AddComment;

use strict;

use warnings;

use base 'MT::App::CMS';



sub init {
	my $app = shift;
	$app->SUPER::init(@_) or return;
	$app->add_methods(
		'add_comment' => \&add_comment,
		'add_ping' => \&add_ping,
	);
	$app->{default_mode} = 'default';
	$app;
}


sub default {
	print "Content-Type: text/html\n\n";
	print "Default mode works\n";
}



####################
### MAIN METHODS ###
####################

# Add new comment
sub add_comment {
    my $app = shift;
    my $q = $app->param;
    my $author = $app->user;
    my $perms = $app->{perms}
        or return $app->error($app->translate("No permissions"));

    my $entry = MT::Entry->load($q->param('id'))
        || return $app->error($app->translate("No such entry."));
    return $app->error($app->translate("Permission denied."))
        unless $perms->can_edit_entry($entry, $author);

    use MT::Comment;
    my $comment = MT::Comment->new;
    $comment->blog_id($entry->blog_id);
    $comment->entry_id($entry->id);
    $comment->author($q->param('author'));
    $comment->email($q->param('email'));
    $comment->url($q->param('url'));
    $comment->text($q->param('text'));
    $comment->ip($app->remote_ip);
    $comment->approve
      if $q->param('status') eq 'publish';

    my $co = $q->param('created_on_manual');
    if ($co) {
        unless ($co =~
            m!(\d{4})-(\d{1,2})-(\d{1,2})\s+(\d{1,2}):(\d{2})(?::(\d{2}))?!) {
            return $app->error($app->translate(
                "Invalid date '[_1]'; authored on dates must be in the format YYYY-MM-DD HH:MM:SS.", $co));
        }
        my $s = $6 || 0;
        return $app->error($app->translate(
            "Invalid date '[_1]'; authored on dates should be real dates.", $co))
            if ($s > 59 || $s < 0  || $5 > 59 || $5 < 0 || $4 > 23 || $4 < 0
                || $2 > 12 || $2 < 1 || $3 < 1
                || (MT::Util::days_in($2, $1) < $3 && !MT::Util::leap_day($0, $1, $2)));
        my $ts = sprintf "%04d%02d%02d%02d%02d%02d", $1, $2, $3, $4, $5, $s;
        $comment->created_on($ts);
    }

    $comment->save
    or die $comment->errstr;

    if ($q->param('status') eq 'publish' && ($entry->status || 0) == MT::Entry::RELEASE()) {
        require MT::Blog;
        my $blog = MT::Blog->load($entry->blog_id);

        $app->rebuild_entry(Entry => $entry)
            or return $app->error($app->translate(
                                  "Rebuild failed: [_1]", $app->errstr));
        MT::Util::start_background_task(sub {
            $app->rebuild_indexes( Blog => $blog )
                or return $app->errtrans("Rebuild failed: [_1]",
                                         $app->errstr);
        });
    }

    return $app->redirect($app->uri('mode' => 'view', 
        args => { 'tab' => 'comments', '_type' => 'entry', id => $entry->id,
        blog_id => $entry->blog_id, saved => 1}));
}

# Add new ping
sub add_ping {
    my $app = shift;
    my $q = $app->param;
    my $author = $app->user;
    my $perms = $app->{perms}
        or return $app->error($app->translate("No permissions"));

    my $entry = MT::Entry->load($q->param('id'))
        || return $app->error($app->translate("No such entry."));
    return $app->error($app->translate("Permission denied."))
        unless $perms->can_edit_entry($entry, $author);

    use MT::Trackback;
    my $tb = MT::Trackback->load({ entry_id => $entry->id });
    return $app->error($app->translate("Permission denied."))
        unless $tb;

    use MT::TBPing;
    my $ping = MT::TBPing->new;
    $ping->blog_id($entry->blog_id);
    $ping->tb_id($tb->id);
    $ping->excerpt($q->param('excerpt'));
    $ping->blog_name($q->param('blog_name'));
    $ping->title($q->param('title'));
    $ping->source_url($q->param('source_url'));
    $ping->ip($app->remote_ip);
    $ping->approve
      if $q->param('status') eq 'publish';

    my $co = $q->param('created_on_manual');
    if ($co) {
        unless ($co =~
            m!(\d{4})-(\d{1,2})-(\d{1,2})\s+(\d{1,2}):(\d{2})(?::(\d{2}))?!) {
            return $app->error($app->translate(
                "Invalid date '[_1]'; authored on dates must be in the format YYYY-MM-DD HH:MM:SS.", $co));
        }
        my $s = $6 || 0;
        return $app->error($app->translate(
            "Invalid date '[_1]'; authored on dates should be real dates.", $co))
            if ($s > 59 || $s < 0  || $5 > 59 || $5 < 0 || $4 > 23 || $4 < 0
                || $2 > 12 || $2 < 1 || $3 < 1
                || (MT::Util::days_in($2, $1) < $3 && !MT::Util::leap_day($0, $1, $2)));
        my $ts = sprintf "%04d%02d%02d%02d%02d%02d", $1, $2, $3, $4, $5, $s;
        $ping->created_on($ts);
    }

    $ping->save
    or die $ping->errstr;

    if ($q->param('status') eq 'publish' && ($entry->status || 0) == MT::Entry::RELEASE()) {
        require MT::Blog;
        my $blog = MT::Blog->load($entry->blog_id);

        $app->rebuild_entry(Entry => $entry)
            or return $app->error($app->translate(
                                  "Rebuild failed: [_1]", $app->errstr));
        MT::Util::start_background_task(sub {
            $app->rebuild_indexes( Blog => $blog )
                or return $app->errtrans("Rebuild failed: [_1]",
                                         $app->errstr);
        });
    }

    return $app->redirect($app->uri('mode' => 'view', 
        args => { 'tab' => 'pings', '_type' => 'entry', id => $entry->id,
        blog_id => $entry->blog_id, saved => 1}));
}


1;
